#!/bin/bash
#
# Copyright (C) 2015 DECOIT GmbH
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

###
# Dieses Icinga-Plugin nutzt Nmap um offene Ports an Systemen in einem Netz zu ermitteln und prüft
# nach einem festgelegten Zeitintervall ob Veränderungen aufgereten sich. Die Ergebnisse werden mittels send_nsca
# an Icinga geschickt.
###

#Parameter für send_nsca festlegen.
HOSTNAME="iMonitor-Sensors"
SERVICE_DESCRIPTION="Nmap Alerts"
ICINGA_HOST=10.240.1.20

#Parameter für Nmap festlegen.
IP_RANGE=10.240.1.0/24
INTERVALL=10

function start_plugin(){

	start_init_scan
	while true; do
		convert_results
		sleep $INTERVALL
	done

}

# Nmap-Scan zum Ermitteln der offenen Ports. Die Ergebnisse werden in einer temporären Datei gespeichert.
function start_init_scan(){

	/usr/bin/nmap $IP_RANGE | grep -oE "[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}|[0-9]{1,5}\/tcp" | cut -d "/" -f 1 | tr "\n" " " > nmap_results

}

# Auslesen der IP-Adressen und Ports aus der temporären Datei.
function convert_results(){

	declare -a array
	j=0
	URLFILE=nmap_results
	for i in `cat $URLFILE`; do
		array[$j]="$i"
		j=$(($j+1))
	done
	extract_data

}

# Für jede gefundene IP-Adresse wird ein SCAN mit den entsprechenden Ports angestoßen.
function extract_data(){

	HOST_IN_CACHE=false
	for i in  ${array[@]}; do
	        if [[ $i =~ [0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3} ]] && [[ "$HOST_IN_CACHE" == "false" ]]; then
			HOST=$i
			HOST_IN_CACHE=true
		else 
			if [[ $i =~ [0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3} ]] && [[ "$HOST_IN_CACHE" == "true" ]]; then
				start_scan
				HOST=$i
				PORTS=""
			fi
		fi
		
		if ! [[ $i =~ [0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3} ]]; then
			PORTS="${PORTS}${i},"
		fi	
	done
	start_scan
	PORTS=""

}

# Jedes System wird auf die gefundenen/festgelegten Ports überprüft.
function start_scan(){

	OUTPUT=`./check_ports.pl --nmap /usr/bin/nmap -i $HOST -p $PORTS`
	split_output

}

# Die Ergebnisse werden gefiltert und in Variablen gespeichert.
function split_output(){

	TIME=`date +%Y-%m-%d" "%H:%M:%S`
	STATUS=`echo $OUTPUT | cut -d';' -f1 | cut -d',' -f1`
	ADDRESS=`echo $OUTPUT | cut -d';' -f1 | cut -d',' -f2 |  cut -d' ' -f3`
	ALLOWED=`echo $OUTPUT | cut -d';' -f3 | cut -d' ' -f3`
	SCANNED=`echo $OUTPUT | cut -d';' -f2| cut -d' ' -f3`
	RESULT=`echo $OUTPUT | cut -d';' -f4 | cut -d' ' -f3`

	build_alert

}

# Aus den Variablen wird die Meldung an Icinga vorbereitet.
function build_alert(){

	ALERT="

		{
		\"type\": \"Nmap\",
		\"timestamp\": \"${TIME}\",
		\"status\": \"${STATUS}\",
		\"ipsrc\": \"10.240.1.40\",
		\"ipdest\": \"${ADDRESS}\",
		\"protocol\": \"TCP\",
		\"message\": \"${RESULT}\",
		\"class\": \"\",
		\"data\": 
				{
					\"ports-allowed\": \"${ALLOWED}\",
					\"ports-scanned\": \"${SCANNED}\"
				}
			  
		}
	     "

	if [ $RESULT ]
		then	start_nsca
	fi

}

# Die Meldung wird an den send_nsca übergeben. Dieser schickt sie an den Icinga-Host.
function start_nsca(){

	SERVICE_STATE="0"
	/bin/echo -e "$HOSTNAME+$SERVICE_DESCRIPTION+$SERVICE_STATE+"$ALERT"\n" | send_nsca -b -H $ICINGA_HOST -d + -c /etc/send_nsca.cfg 1>/dev/null &

}

# Startet das Plugin.
start_plugin
